
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#define SIZE  6
	/* field size */
#define MAXSTATES  (1024*1024*8-16)
#define MAXOPENS  (1024*1024*1-16)
	/* max size of state space */
#define MAXPOS  8
	/* max # of possibilities in a single row */

#define CH_EMPTY  '.'
#define CH_FULL  '*'

typedef unsigned char nrmoves_t;
#define INFINITY  255

typedef char state_t [SIZE+2][SIZE+2];
	/* town map [0..5][0..5] */
typedef char row_t [SIZE+2];
typedef char* rowptr_t;
	/* one row or column [1..6]=data, [0]=[7]=CH_FULL (sentinel) */


/*
  Which vehicles are there in every single row and column.
  [0..5] rows 0-5
  [6..11] columns 0-5
*/
int vcnt [2*SIZE];   /* number of vehicles (0 to 3) */
char vchr [2*SIZE][3];   /* corresponding lowercase characters */
int vlen [2*SIZE][3];   /* vehicle lengths */
int vpoc [2*SIZE];   /* number of possible configurations */
row_t vpos [2*SIZE][MAXPOS];   /* the configurations */


/*
  Trivial work with rows and columns
*/

void rmcross_row (rowptr_t rw, char fch)
{
	int i;
	/* replace all crossing vehicles with fch character */
	rw[0] = rw[SIZE+1] = CH_FULL;
	for ( i = 1;  i <= SIZE;  ++i )
		if ( islower (rw[i]) && rw[i] != rw[i-1] && rw[i] != rw[i+1] )
			rw[i] = fch;
}

void clear_row (rowptr_t rw)
{
	int i;
	for ( i = 1;  i <= SIZE;  ++i )
		if ( islower (rw[i]) )  rw[i] = '.';
}

void empty_state (state_t* st)
{
	int i, j;
	for ( i = 0;  i < SIZE;  ++i )
		for ( j = 0;  j < SIZE;  ++j )
			(*st)[i][j] = CH_EMPTY;
}

void copy_row (rowptr_t dst, rowptr_t src)
{
	memcpy (dst, src, SIZE+2);
}

void select_row (int idx, state_t* st, rowptr_t res)
{
	int i;
	for ( i = 0;  i < SIZE;  ++i )
		res[i+1] = (*st)[idx][i];
}

void select_col (int idx, state_t* st, rowptr_t res)
{
	int i;
	for ( i = 0;  i < SIZE;  ++i )
		res[i+1] = (*st)[i][idx];
}

void putback_row (int idx, state_t* st, rowptr_t rw)
{
	int i;
	for ( i = 0;  i < SIZE;  ++i )
		if ( islower (rw[i+1]) )
			(*st)[idx][i] = rw[i+1];
}

void putback_col (int idx, state_t* st, rowptr_t rw)
{
	int i;
	for ( i = 0;  i < SIZE;  ++i )
		if ( islower (rw[i+1]) )
			(*st)[i][idx] = rw[i+1];
}


/*
  Scan for vehicles. Fill `vxxx' globals.
*/

void loc_one_pos (int vi, int p0, int p1, int p2)
{
	row_t r;
	int i;

	if ( vcnt[vi] < 1 )  return;
	if ( vcnt[vi] < 2 && p1 > p0 )  return;
	if ( vcnt[vi] < 3 && p2 > p1 )  return;

	for ( i = 0;  i <= SIZE+1;  ++i )  r[i] = CH_EMPTY;
	rmcross_row (r, CH_EMPTY);
	for ( i = p0;  i < p0 + vlen[vi][0];  ++i )
	{
		if ( r[i] != CH_EMPTY )  return;
		r[i] = vchr[vi][0];
	}
	if ( vcnt[vi] >= 2 )
		for ( i = p1;  i < p1 + vlen[vi][1];  ++i )
		{
			if ( r[i] != CH_EMPTY )  return;
			r[i] = vchr[vi][1];
		}
	if ( vcnt[vi] >= 3 )
		for ( i = p2;  i < p2 + vlen[vi][2];  ++i )
		{
			if ( r[i] != CH_EMPTY )  return;
			r[i] = vchr[vi][2];
		}
	copy_row (vpos[vi][vpoc[vi]++], r);
}

void locate_row (int vi, rowptr_t rw)
{
	int i, j, k;

	rmcross_row (rw, CH_EMPTY);

	vcnt[vi] = 0;
	for ( j = 1;  j <= SIZE;  ++j )
		if ( islower (rw[j]) )
		{
			if ( vcnt[vi] == 0 || rw[j-1] != rw[j] )
			{
				vchr[vi][vcnt[vi]] = rw[j];
				vlen[vi][vcnt[vi]++] = 0;
			}
			++vlen[vi][vcnt[vi]-1];
		}

	vpoc[vi] = 0;
	for ( i = 1;  i < SIZE;  ++i )
		for ( j = i;  j < SIZE;  ++j )
			for ( k = j;  k < SIZE;  ++k )
				loc_one_pos (vi, i, j, k);
}

void locate_vehicles (state_t* st)
{
	int i;
	row_t r;
	for ( i = 0;  i < SIZE;  ++i )
	{
		select_row (i, st, r);
		locate_row (i, r);
	}
	for ( i = 0;  i < SIZE;  ++i )
	{
		select_col (i, st, r);
		locate_row (SIZE + i, r);
	}
}


/*
  Return all of the possible configurations of vehicles.
  Non-empty (CH_FULL) squares cannot be used!
*/
row_t all_pos [MAXPOS];

int gen_pos (rowptr_t rw, int vi)
{
	int i, j, cnt = 0;
	row_t r2;

	copy_row (r2, rw);
	clear_row (r2);
	for ( i = 0;  i < vpoc[vi];  ++i )
	{
		for ( j = 1;  j <= SIZE;  ++j )
			if ( islower (vpos[vi][i][j]) )
			{
				if ( r2[j] != CH_EMPTY )  break;
				all_pos[cnt][j] = vpos[vi][i][j];
			}
			else
				all_pos[cnt][j] = r2[j];
		if ( j > SIZE )  ++cnt;
	}
	return cnt;
}


/*
  Find the position in the list of all possible configurations.
*/

int find_pos (rowptr_t rw, int vi, int *pcnt)
{
	int i, j;
	*pcnt = gen_pos (rw, vi);
	if ( *pcnt == 0 )  { *pcnt = 1; return 0; }
	for ( i = 0;  i < *pcnt;  ++i )
	{
		for ( j = 1;  j <= SIZE;  ++j )
			if ( rw[j] != all_pos[i][j] )  break;
		if ( j > SIZE )  return i;
	}
	fprintf (stderr, "ERROR: cannot find pos\n");
	return 0;
}


/*
  Encode state to a unique number.
*/
static int max_code_value = 0;
int state_encode (state_t* st)
{
	int mul = 1, val = 0;
	int i, p, cnt;
	row_t r;

	for ( i = 0;  i < SIZE;  ++i )
	{
		select_row (i, st, r);
		rmcross_row (r, CH_EMPTY);
		p = find_pos (r, i, &cnt);
		val += mul * p;
		mul *= cnt;
	}

	for ( i = 0;  i < SIZE;  ++i )
	{
		select_col (i, st, r);
		rmcross_row (r, CH_FULL);
		p = find_pos (r, SIZE + i, &cnt);
		val += mul * p;
		mul *= cnt;
	}

	if ( val > max_code_value )  max_code_value = val;

	if ( val > MAXSTATES )
	{
		fprintf (stderr, "ERROR: state code too big!!!\n");
		val = 0;
	}
	return val;
}

/*
  Decode state from its unique number.
*/
void state_decode (int code, state_t* st)
{
	int i, cnt;
	row_t r;

	empty_state (st);

	for ( i = 0;  i < SIZE;  ++i )
	{
		select_row (i, st, r);
		rmcross_row (r, CH_EMPTY);
		cnt = gen_pos (r, i);
		if ( cnt == 0 )  continue;
		putback_row (i, st, all_pos[code % cnt]);
		code /= cnt;
	}

	for ( i = 0;  i < SIZE;  ++i )
	{
		select_col (i, st, r);
		rmcross_row (r, CH_FULL);
		cnt = gen_pos (r, SIZE + i);
		if ( cnt == 0 )  continue;
		putback_col (i, st, all_pos[code % cnt]);
		code /= cnt;
	}
}


/*
  STATE SPACE
*/
nrmoves_t best [MAXSTATES];
int qs [MAXOPENS], qb, qe;
int solution;


void enqueue (state_t* st, int dist)
{
	int i;
	i = state_encode (st);
	if ( best[i] <= dist )  return;
	best[i] = dist;
	qs[qe++ % MAXOPENS] = i;
	for ( i = 0;  i < SIZE;  ++i )
		if ( (*st)[i][SIZE-1] == 'x' && solution > dist )
			solution = dist;

#ifdef DEBUG
	printf ("Distance: %d\n", dist);
	{
		int j;
		for ( i = 0;  i < SIZE;  ++i )
			for ( j = 0;  j < SIZE+1;  ++j )
				putchar (j < SIZE ? (*st)[i][j] : '\n');
		putchar ('\n');
	}
#endif
}


void expand_row (state_t* st, int dist, char** chp)
{
	char ch;
	int i, j, k, x;
	for ( i = 1;  i <= SIZE-2;  ++i )
		if ( *chp[i] == CH_EMPTY )
		{
			for ( j = i;  *chp[j] == CH_EMPTY && j <= SIZE-1;  ++j ) {}
			for ( k = j;  *chp[k] == *chp[j] && k <= SIZE;  ++k ) {}
			if ( k-j > 1 )
			{
				/* move car [j..k-1] to [i] */
				ch = *chp[j];
				for ( x = 0;  x < k-j;  ++x )  *chp[j+x] = CH_EMPTY;
				for ( x = 0;  x < k-j;  ++x )  *chp[i+x] = ch;
				enqueue (st, dist);
				for ( x = 0;  x < k-j;  ++x )  *chp[i+x] = CH_EMPTY;
				for ( x = 0;  x < k-j;  ++x )  *chp[j+x] = ch;
			}
		}
	for ( i = SIZE;  i >= 3;  --i )
		if ( *chp[i] == CH_EMPTY )
		{
			for ( j = i;  *chp[j] == CH_EMPTY && j >= 2;  --j ) {}
			for ( k = j;  *chp[k] == *chp[j] && k >= 1;  --k ) {}
			if ( j-k > 1 )
			{
				/* move car [k+1..j] to [..i] */
				ch = *chp[j];
				for ( x = 0;  x < j-k;  ++x )  *chp[j-x] = CH_EMPTY;
				for ( x = 0;  x < j-k;  ++x )  *chp[i-x] = ch;
				enqueue (st, dist);
				for ( x = 0;  x < j-k;  ++x )  *chp[i-x] = CH_EMPTY;
				for ( x = 0;  x < j-k;  ++x )  *chp[j-x] = ch;
			}
		}
}

void expand (state_t* st, int dist)
{
	int i, j;
	char chf = CH_FULL;
	char* chp [SIZE+2];
	chp[0] = chp[SIZE+1] = &chf;
	for ( i = 0;  i < SIZE;  ++i )
	{
		for ( j = 0;  j < SIZE;  ++j )
			chp[j+1] = &(*st)[i][j];
		expand_row (st, dist+1, chp);
	}
	for ( i = 0;  i < SIZE;  ++i )
	{
		for ( j = 0;  j < SIZE;  ++j )
			chp[j+1] = &(*st)[j][i];
		expand_row (st, dist+1, chp);
	}
}



/*
  MAIN PROGRAM
*/

int main (void)
{
	int veh, scen = 0;
	for ( ; ; )
	{
		int i;
		state_t beg;

		if ( scanf ("%d\n", &veh) < 1 )  break;
		if ( veh < 1 )  break;
		for ( i = 0;  i < SIZE;  ++i )
			scanf ("%s\n", beg[i]);
		locate_vehicles (&beg);

		for ( i = 0;  i < MAXSTATES;  ++i )
			best[i] = INFINITY;
		solution = INFINITY;
		qb = qe = 0;
		enqueue (&beg, 0);
		while ( qb < qe && solution == INFINITY )
		{
			int s = qs[qb++ % MAXOPENS];
			if ( qe-qb >= MAXOPENS )
				fprintf (stderr, "ERROR: Too many open states!\n");
			state_decode (s, &beg);
			expand (&beg, best[s]);
		}
		if ( solution == 0 )  solution = 1;
		if ( solution < INFINITY )
			printf ("Scenario #%d requires %d moves.\n",
					++scen, solution);
		else
			printf ("You are trapped in scenario #%d.\n",
					++scen);
	}
	/* printf ("Max Code Value: %d\n", max_code_value); */
	return 0;
}

